#!/bin/sh
# ----------------------------------------------------------------------------
# /var/install/config.d/freeradius - FREERADIUS configuration
#
# Creation   : 2010-02-19 stm
# Last update: $Id: freeradius.sh 551 2011-11-14 18:58:19Z stm $
#
# Copyright (c) 2010 Stephan Manske, eisfair-develop(at)stephan(dot)manske-net(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> `pwd`/freeradius-trace$$.log
#set -x

# ----------------------------------------------------------------------------
# Config Dateien
# ----------------------------------------------------------------------------

callDir=`pwd`

# include eislib
. /var/install/include/eislib
. /var/install/include/configlib

# include config
CONFIG_FILE=/etc/config.d/freeradius
. ${CONFIG_FILE}


pgmname=`basename $0`

# set packageName
packageName=freeradius

certs_path='/etc/raddb/certs'
settingsPath='/etc/raddb'

usersFile='users.eis' 
clientsFile='clients.eis'
rejectFile='reject_delay.eis'
caCertFile='ca.cnf'
serverCertFile='server.cnf'

sslconf_sik=${OPENSSL_CONF} 

 # set OpenSSL environment variable to prevent an error message 

 export OPENSSL_CONF=${certs_path}/ca.cnf

caCert='ca.der'
#serverCert='server.pem'


crontab_path=/var/cron/etc/root
generate_crontab=${crontab_path}/freeradius



# ----------------------------------------------------------------------------
# create users.eis
# ----------------------------------------------------------------------------

create_users_conf ()
{

    cd ${settingsPath}

    rm -f ${usersFile}
    mecho "Create user/supplicant list ..."

    cat >${usersFile} <<-EOF
#--------------------------------------------------------------------
#  config file generated by ${packageName}
#
#  Do not edit this file, edit ${CONFIG_FILE}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
#--------------------------------------------------------------------

EOF

# for every active user

    eval ncnt='${FREERADIUS_USER_N}'
    if [ ! -z "$ncnt" ]
    then
        xn="1"
        while [ $xn -le $ncnt ]
        do
            eval tempname='$FREERADIUS_USER_'$xn'_NAME'
            eval temppass='$FREERADIUS_USER_'$xn'_PASS'
            eval tempactive='$FREERADIUS_USER_'$xn'_ACTIVE'
            eval tempcert='$FREERADIUS_USER_'$xn'_CERT_ONLY'
           

            if [ ${tempactive} = 'yes' ]
            then
                if [ ${tempcert} = 'yes' ]
                then
                    echo "\"${tempname}\"       " >>${usersFile} 
                else
                    echo "\"${tempname}\"     Cleartext-Password := ${temppass}" >>${usersFile} 
                fi


                # for every custom parameter
                
                eval ncnt2='$FREERADIUS_USER_'$xn'_CUSTOM_N'
                if [ ! -z "$ncnt2" ]
                then
                    xn2="1"
                    while [ $xn2 -le $ncnt2 ]
                    do
                        eval tempparam='$FREERADIUS_USER_'$xn'_CUSTOM_'$xn2
                        echo "                ${tempparam}" >>${usersFile} 
            
                        xn2=`expr $xn2 + 1`
                    done
                fi

                               
                echo >>${usersFile} 
            fi

            xn=`expr $xn + 1`
        done
    fi

    chmod 0640 ${usersFile}
}


# ----------------------------------------------------------------------------
# Create clients.eis
# ----------------------------------------------------------------------------

create_clients_conf ()
{

    cd ${settingsPath}
    
    rm -f ${clientsFile}
    mecho "Create clients list ..."

    cat >${clientsFile} <<-EOF
#--------------------------------------------------------------------
#  config file generated by ${packageName}
#
#  Do not edit this file, edit ${CONFIG_FILE}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
#--------------------------------------------------------------------

EOF

# for every active client

    eval ncnt='${FREERADIUS_CLIENT_N}'
    if [ ! -z "$ncnt" ]
    then
        xn="1"
        while [ $xn -le $ncnt ]
        do
            eval tempname='$FREERADIUS_CLIENT_'$xn'_NAME'
            eval temppass='$FREERADIUS_CLIENT_'$xn'_SECRET'
            eval tempip='$FREERADIUS_CLIENT_'$xn'_IP'
            eval tempnastype='$FREERADIUS_CLIENT_'$xn'_NASTYPE'
            eval tempactive='$FREERADIUS_CLIENT_'$xn'_ACTIVE'

            if [ ${tempactive} = 'yes' ]
            then
                echo "client ${tempname} {" >>${clientsFile} 
                
                if [ ! ${tempip} = '' ]
                then
                    echo "      ipaddr = ${tempip}" >>${clientsFile} 
                fi
                
                echo "      secret = ${temppass}" >>${clientsFile} 
                echo "      require_message_authenticator = no" >>${clientsFile} 
                echo "      nastype = ${tempnastype}" >>${clientsFile} 
                echo "}" >>${clientsFile} 
                echo >>${clientsFile} 
            fi

            xn=`expr $xn + 1`
        done
    fi

    chmod 0640 ${clientsFile}
}

# ----------------------------------------------------------------------------
# reject_delay settings
# ----------------------------------------------------------------------------

create_reject_conf ()
{
    cd ${settingsPath}

    rm -f ${rejectFile}
    mecho "Create some other stuff ..."

    cat >${rejectFile} <<-EOF
#--------------------------------------------------------------------
#  config file generated by ${packageName}
#
#  Do not edit this file, edit ${CONFIG_FILE}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
#--------------------------------------------------------------------

reject_delay = ${FREERADIUS_REJECT_DELAY}

EOF


    chmod 0640 ${rejectFile}
}


#----------------------------------------------------------------------------------------
# send new certificate via mail
#----------------------------------------------------------------------------------------

send_certmail ()
{

    errflag=0
    
    if [ ! -f ca.der ]
    then
        mecho -error "Error: couldn't find certificate ${certs_path}/ca.der!"
        mecho -error "       Function aborted!"
        mecho
        errflag=1
    fi

    if [ ! -f ca.pem ]
    then
        mecho -error "Error: couldn't find certificate ${certs_path}/ca.pem!"
        mecho -error "       Function aborted!"
        mecho
        errflag=1
    fi

    if [ ${errflag} -eq 0 ]
    then
        # go on...
        mecho
        mecho -info "Send  certificates ${certs_path}/ca.der ${certs_path}/ca.pem to admin ..."


        mail_addr="${FREERADIUS_CERT_MAIL}"

        # send ca and server certificate
        {
            echo "Attached you will find the certificate of the Certification Authority (CA)."
            echo "Please open the attachments:"
            echo "ca.der (should work for windows) or ca.pem"
            echo "Please follow the import instruction from your OS."
        } | /usr/local/bin/send_file_from_cmd.sh -f "CA master <postmaster>"  \
            -s "Your CA certificates - check attachements" -t "${mail_addr}" \
            -b "${certs_path}/ca.der ${certs_path}/ca.pem"


        echo
    fi
}

# ----------------------------------------------------------------------------
# make new ca and server certificates
# ----------------------------------------------------------------------------

create_ca_certs_conf ()
{
    cd ${certs_path}

    rm -f ${caCertFile}
    mecho "Create ca certificate config ..."

    cat >${caCertFile} <<-EOF
#--------------------------------------------------------------------
#  config file generated by ${packageName} version: ${VERSION}
#
#  Do not edit this file, edit ${CONFIG_FILE}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
#--------------------------------------------------------------------

[ ca ]
default_ca              = CA_default

[ CA_default ]
dir                     = ./
certs                   = \$dir
crl_dir                 = \$dir/crl
database                = \$dir/index.txt
new_certs_dir           = \$dir
certificate             = \$dir/ca.pem
serial                  = \$dir/serial
crl                     = \$dir/crl.pem
private_key             = \$dir/ca.key
RANDFILE                = \$dir/.rand
name_opt                = ca_default
cert_opt                = ca_default
default_days            = ${FREERADIUS_CERT_CA_TIME}
default_crl_days        = 30
default_md              = md5
preserve                = no
policy                  = policy_match

[ policy_match ]
countryName             = match
stateOrProvinceName     = match
organizationName        = match
organizationalUnitName  = optional
commonName              = supplied
emailAddress            = optional

[ policy_anything ]
countryName             = optional
stateOrProvinceName     = optional
localityName            = optional
organizationName        = optional
organizationalUnitName  = optional
commonName              = supplied
emailAddress            = optional

[ req ]
prompt                  = no
distinguished_name      = certificate_authority
default_bits            = 2048
input_password          = whatever
output_password         = whatever
x509_extensions         = v3_ca

[certificate_authority]
countryName             = ${FREERADIUS_CERT_COUNTRY}
stateOrProvinceName     = Somewhere
localityName            = Somewhere
organizationName        = ${FREERADIUS_CERT_ORGNAME}
organizationalUnitName  = Radius_Managment
emailAddress            = ${FREERADIUS_CERT_MAIL}
commonName              = ${FREERADIUS_CERT_CA_NAME}

[v3_ca]
subjectKeyIdentifier    = hash
authorityKeyIdentifier  = keyid:always,issuer:always
basicConstraints        = CA:true

EOF

   chmod 0640 ${caCertFile}

}


create_server_certs_conf ()
{
    cd ${certs_path}

    rm -f ${serverCertFile}
    mecho "Create server certificate config ..."

    cat >${serverCertFile} <<-EOF
#--------------------------------------------------------------------
#  config file generated by ${packageName} version: ${VERSION}
#
#  Do not edit this file, edit ${CONFIG_FILE}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
#--------------------------------------------------------------------

[ ca ]
default_ca              = CA_default

[ CA_default ]
dir                     = ./
certs                   = \$dir
crl_dir                 = \$dir/crl
database                = \$dir/index.txt
new_certs_dir           = \$dir
certificate             = \$dir/server.pem
serial                  = \$dir/serial
crl                     = \$dir/crl.pem
private_key             = \$dir/server.key
RANDFILE                = \$dir/.rand
name_opt                = ca_default
cert_opt                = ca_default
default_days            = ${FREERADIUS_CERT_SERVER_TIME}
default_crl_days        = 30
default_md              = md5
preserve                = no
policy                  = policy_match

[ policy_match ]
countryName             = match
stateOrProvinceName     = match
organizationName        = match
organizationalUnitName  = optional
commonName              = supplied
emailAddress            = optional

[ policy_anything ]
countryName             = optional
stateOrProvinceName     = optional
localityName            = optional
organizationName        = optional
organizationalUnitName  = optional
commonName              = supplied
emailAddress            = optional

[ req ]
prompt                  = no
distinguished_name      = server
default_bits            = 2048
input_password          = whatever
output_password         = whatever

[server]
countryName             = ${FREERADIUS_CERT_COUNTRY}
stateOrProvinceName     = Somewhere
localityName            = Somewhere
organizationName        = ${FREERADIUS_CERT_ORGNAME}
organizationalUnitName  = Radius_Managment
emailAddress            = ${FREERADIUS_CERT_MAIL}
commonName              = ${FREERADIUS_CERT_SERVER_NAME}

EOF

   chmod 0640 ${serverCertFile}

}


generate_crl ()
{
    PASSWORD_CA=`grep output_password ca.cnf | sed 's/.*=//;s/^ *//'`
    openssl ca -gencrl -out ./crl/cr1.pem -config ./ca.cnf -key ${PASSWORD_CA}

    ln -sf ../ca.pem ./crl/ca.pem
    freeradius-c_rehash ./crl
}

#-------------------------------------------------------------------
# main function to create certificates - but no user certs!
#-------------------------------------------------------------------

create_certs ()
{
    cd ${certs_path}


# are there ca/server certificates allreay?

ca_cert_update=0
#server_cert_update=0


# if ca certificate is away - we have to rebuild the whole cert infrastructure!


if [ ! -f ${certs_path}/${caCert} ]
then
    ca_cert_update=1
#    server_cert_update=1
    mecho -info "CA certificate is not here, will create new one."
    anykey
fi

#if [ [ ! -f ${certs_path}/${serverCert} ] && [ ${server_cert_update} -ne 1 ] ]
#then
#    server_cert_update=1
#    mecho -info "Server certificate is not here, will create new one."
#    anykey
#fi

if [ ${FREERADIUS_MAKE_NEW_CERTS_NOW} = "yes" ]
then

# make FREERADIUS_MAKE_NEW_CERTS_NOW = no in config-file  
   
    sed -e "s/^FREERADIUS_MAKE_NEW_CERTS_NOW.*=.*yes.*$/FREERADIUS_MAKE_NEW_CERTS_NOW='no'/" ${CONFIG_FILE} > ${CONFIG_FILE}.new
    rm ${CONFIG_FILE}
    mv ${CONFIG_FILE}.new ${CONFIG_FILE}

    if [ $ca_cert_update -eq 0 ]
    then
        mecho -warn "You started an (re)build of the certification structure!"
        answer=`/var/install/bin/ask "Do you really realy want to delete all the current certificates? All means: CA, server and users! This is needed for a new certificate structure!" "no"`
        mecho ${answer}
    
        if [ ${answer} = "yes" ]
        then
            ca_cert_update=1
#            server_cert_update=1
        else
            mecho "(Re)Build aborted - FREERADIUS_MAKE_NEW_CERTS_NOW reset to no."    
        fi
    fi
fi    

if [ $ca_cert_update -eq 1 ]
then

    mecho -warn "The server and all user certificates used by RADIUS service (if created before) must and will be deleted now!"
    mecho -warn "You have to create new user certificates and"
    mecho -warn "users have to import new CA and server certificates"
    mecho -warn "and their new own user certificate."

    anykey

    freeradius-make destroycerts
    
    rm -f ${certs_path}/crl/*
    rm -f ${certs_path}/archive/*

    # leere die SERIAL-Variablen
    sed -r "s/^FREERADIUS_USER_(.*)_CERT_SERIAL.*$/FREERADIUS_USER_\1_CERT_SERIAL=''/" ${CONFIG_FILE} > ${CONFIG_FILE}.new
    rm -f ${CONFIG_FILE}
    mv ${CONFIG_FILE}.new ${CONFIG_FILE}

    create_ca_certs_conf
    create_server_certs_conf
    
    freeradius-make all
    
      
    anykey
    
    # mail certificates?
    
    if [ ${FREERADIUS_CERT_MAILME} = "yes" ]
    then
        send_certmail
    fi

fi

#if [ $server_cert_update -eq 1 ]
#then
#
#    mecho -warn "users have to import the new server certificate."
#
#    create_server_certs_conf
#    
#    freeradius-make server
#    
#    anykey
#    
#    # mail certificates?
#    
#    if [ ${FREERADIUS_CERT_MAILME} = "yes" ]
#    then
#        send_certmail
#    fi
#
#fi

}


#----------------------------------------------------------------------------------------
# add cron job
#----------------------------------------------------------------------------------------
add_cron_job ()
{
    mecho "creating cron job ..."

    # check for cron directory
    if [ ! -d ${crontab_path} ]
    then
        mkdir -p ${crontab_path}
    fi

    # write file
    {
        echo "#--------------------------------------------------------------------"
        echo "#  cron ${pgmname} file generated by ${pgmname}"
        echo "#"
        echo "#  Do not edit this file, edit ${CONFIG_FILE}"
        echo "#  Creation Date: ${EISDATE} Time: ${EISTIME}"
        echo "#--------------------------------------------------------------------"

        echo "10 8 * * 5 /var/install/bin/freeradius-renew-crl"
        echo
    } > ${generate_crontab}

    # update crontab file
    /var/install/config.d/cron.sh -quiet
}

#----------------------------------------------------------------------------------------
# delete cron job
# $1 - quiet option
#----------------------------------------------------------------------------------------
delete_cron_job ()
{
    if [ "$1" != "-quiet" ]
    then
        mecho "deleting cron job ..."
    fi

    # check for crontab file
    if [ -f ${generate_crontab} ]
    then
        # delete existing file
        rm -f ${generate_crontab}

        # update crontab file
        /var/install/config.d/cron.sh -quiet
    fi
}



# ----------------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------------

case "$1" in
    del_cronjob)
        delete_cron_job -quiet
        ;;
    *)
        # generate all configuration files
        if [ ${START_FREERADIUS} = "yes" ] 
        then

            create_users_conf
            create_clients_conf
            create_reject_conf
    
            create_certs
    
            generate_crl
    
            add_cron_job

        else
            delete_cron_job
        fi

        ;;
esac

# restore previous OpenSSL environment variable state 

 if [ "${sslconf_sik}" = "" ] 
 then 

 unset OPENSSL_CONF 

 else 

 export OPENSSL_CONF=${sslconf_sik} 

 fi

cd ${callDir}

exit 0
# ----------------------------------------------------------------------------
# End
# ----------------------------------------------------------------------------



